@@ -1,29 +1,27 @@
-from __future__ import print_function
-import redis
 import time
-import sys
-from functools import wraps
 from argparse import ArgumentParser
+from functools import wraps
 
-if sys.version_info[0] == 3:
-    long = int
+import redis
 
 
 def parse_args():
     parser = ArgumentParser()
-    parser.add_argument('-n',
-                        type=int,
-                        help='Total number of requests (default 100000)',
-                        default=100000)
-    parser.add_argument('-P',
-                        type=int,
-                        help=('Pipeline <numreq> requests.'
-                              ' Default 1 (no pipeline).'),
-                        default=1)
-    parser.add_argument('-s',
-                        type=int,
-                        help='Data size of SET/GET value in bytes (default 2)',
-                        default=2)
+    parser.add_argument(
+        "-n", type=int, help="Total number of requests (default 100000)", default=100000
+    )
+    parser.add_argument(
+        "-P",
+        type=int,
+        help=("Pipeline <numreq> requests. Default 1 (no pipeline)."),
+        default=1,
+    )
+    parser.add_argument(
+        "-s",
+        type=int,
+        help="Data size of SET/GET value in bytes (default 2)",
+        default=2,
+    )
 
     args = parser.parse_args()
     return args
@@ -47,18 +45,19 @@ def run():
 def timer(func):
     @wraps(func)
     def wrapper(*args, **kwargs):
-        start = time.clock()
+        start = time.monotonic()
         ret = func(*args, **kwargs)
-        duration = time.clock() - start
-        if 'num' in kwargs:
-            count = kwargs['num']
+        duration = time.monotonic() - start
+        if "num" in kwargs:
+            count = kwargs["num"]
         else:
             count = args[1]
-        print('{} - {} Requests'.format(func.__name__, count))
-        print('Duration  = {}'.format(duration))
-        print('Rate = {}'.format(count/duration))
-        print('')
+        print(f"{func.__name__} - {count} Requests")
+        print(f"Duration  = {duration}")
+        print(f"Rate = {count / duration}")
+        print()
         return ret
+
     return wrapper
 
 
@@ -67,10 +66,9 @@ def set_str(conn, num, pipeline_size, data_size):
     if pipeline_size > 1:
         conn = conn.pipeline()
 
-    format_str = '{:0<%d}' % data_size
-    set_data = format_str.format('a')
+    set_data = "a".ljust(data_size, "0")
     for i in range(num):
-        conn.set('set_str:%d' % i, set_data)
+        conn.set(f"set_str:{i}", set_data)
         if pipeline_size > 1 and i % pipeline_size == 0:
             conn.execute()
 
@@ -83,10 +81,9 @@ def set_int(conn, num, pipeline_size, data_size):
     if pipeline_size > 1:
         conn = conn.pipeline()
 
-    format_str = '{:0<%d}' % data_size
-    set_data = int(format_str.format('1'))
+    set_data = 10 ** (data_size - 1)
     for i in range(num):
-        conn.set('set_int:%d' % i, set_data)
+        conn.set(f"set_int:{i}", set_data)
         if pipeline_size > 1 and i % pipeline_size == 0:
             conn.execute()
 
@@ -100,7 +97,7 @@ def get_str(conn, num, pipeline_size, data_size):
         conn = conn.pipeline()
 
     for i in range(num):
-        conn.get('set_str:%d' % i)
+        conn.get(f"set_str:{i}")
         if pipeline_size > 1 and i % pipeline_size == 0:
             conn.execute()
 
@@ -114,7 +111,7 @@ def get_int(conn, num, pipeline_size, data_size):
         conn = conn.pipeline()
 
     for i in range(num):
-        conn.get('set_int:%d' % i)
+        conn.get(f"set_int:{i}")
         if pipeline_size > 1 and i % pipeline_size == 0:
             conn.execute()
 
@@ -128,7 +125,7 @@ def incr(conn, num, pipeline_size, *args, **kwargs):
         conn = conn.pipeline()
 
     for i in range(num):
-        conn.incr('incr_key')
+        conn.incr("incr_key")
         if pipeline_size > 1 and i % pipeline_size == 0:
             conn.execute()
 
@@ -141,10 +138,9 @@ def lpush(conn, num, pipeline_size, data_size):
     if pipeline_size > 1:
         conn = conn.pipeline()
 
-    format_str = '{:0<%d}' % data_size
-    set_data = int(format_str.format('1'))
+    set_data = 10 ** (data_size - 1)
     for i in range(num):
-        conn.lpush('lpush_key', set_data)
+        conn.lpush("lpush_key", set_data)
         if pipeline_size > 1 and i % pipeline_size == 0:
             conn.execute()
 
@@ -158,7 +154,7 @@ def lrange_300(conn, num, pipeline_size, data_size):
         conn = conn.pipeline()
 
     for i in range(num):
-        conn.lrange('lpush_key', i, i+300)
+        conn.lrange("lpush_key", i, i + 300)
         if pipeline_size > 1 and i % pipeline_size == 0:
             conn.execute()
 
@@ -171,7 +167,7 @@ def lpop(conn, num, pipeline_size, data_size):
     if pipeline_size > 1:
         conn = conn.pipeline()
     for i in range(num):
-        conn.lpop('lpush_key')
+        conn.lpop("lpush_key")
         if pipeline_size > 1 and i % pipeline_size == 0:
             conn.execute()
     if pipeline_size > 1:
@@ -183,12 +179,9 @@ def hmset(conn, num, pipeline_size, data_size):
     if pipeline_size > 1:
         conn = conn.pipeline()
 
-    set_data = {'str_value': 'string',
-                'int_value': 123456,
-                'long_value': long(123456),
-                'float_value': 123456.0}
+    set_data = {"str_value": "string", "int_value": 123456, "float_value": 123456.0}
     for i in range(num):
-        conn.hmset('hmset_key', set_data)
+        conn.hmset("hmset_key", set_data)
         if pipeline_size > 1 and i % pipeline_size == 0:
             conn.execute()
 
@@ -196,5 +189,5 @@ def hmset(conn, num, pipeline_size, data_size):
         conn.execute()
 
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     run()
