@@ -11,3 +11,7 @@ class Document:
 
     def __repr__(self):
         return f"Document {self.__dict__}"
+
+    def __getitem__(self, item):
+        value = getattr(self, item)
+        return value
