@@ -1,5 +1,7 @@
-import pytest
+from contextlib import closing
+from unittest import mock
 
+import pytest
 import redis
 
 from .conftest import skip_if_server_version_lt, wait_for_command
@@ -19,7 +21,6 @@ class TestPipeline:
                 .zadd("z", {"z1": 1})
                 .zadd("z", {"z2": 4})
                 .zincrby("z", 1, "z1")
-                .zrange("z", 0, 5, withscores=True)
             )
             assert pipe.execute() == [
                 True,
@@ -27,7 +28,6 @@ class TestPipeline:
                 True,
                 True,
                 2.0,
-                [(b"z1", 2.0), (b"z2", 4)],
             ]
 
     def test_pipeline_memoryview(self, r):
@@ -122,7 +122,7 @@ class TestPipeline:
             with pytest.raises(redis.ResponseError) as ex:
                 pipe.execute()
             assert str(ex.value).startswith(
-                "Command # 3 (LPUSH c 3) of " "pipeline caused error: "
+                "Command # 3 (LPUSH c 3) of pipeline caused error: "
             )
 
             # make sure the pipe was restored to a working state
@@ -167,7 +167,7 @@ class TestPipeline:
                 pipe.execute()
 
             assert str(ex.value).startswith(
-                "Command # 2 (ZREM b) of " "pipeline caused error: "
+                "Command # 2 (ZREM b) of pipeline caused error: "
             )
 
             # make sure the pipe was restored to a working state
@@ -184,7 +184,7 @@ class TestPipeline:
                 pipe.execute()
 
             assert str(ex.value).startswith(
-                "Command # 2 (ZREM b) of " "pipeline caused error: "
+                "Command # 2 (ZREM b) of pipeline caused error: "
             )
 
             # make sure the pipe was restored to a working state
@@ -287,6 +287,24 @@ class TestPipeline:
             assert unwatch_command is not None
             assert unwatch_command["command"] == "UNWATCH"
 
+    @pytest.mark.onlynoncluster
+    def test_close_is_reset(self, r):
+        with r.pipeline() as pipe:
+            called = 0
+
+            def mock_reset():
+                nonlocal called
+                called += 1
+
+            with mock.patch.object(pipe, "reset", mock_reset):
+                pipe.close()
+                assert called == 1
+
+    @pytest.mark.onlynoncluster
+    def test_closing(self, r):
+        with closing(r.pipeline()):
+            pass
+
     @pytest.mark.onlynoncluster
     def test_transaction_callable(self, r):
         r["a"] = 1
@@ -331,7 +349,7 @@ class TestPipeline:
                 pipe.execute()
 
             assert str(ex.value).startswith(
-                "Command # 1 (LLEN a) of " "pipeline caused error: "
+                "Command # 1 (LLEN a) of pipeline caused error: "
             )
 
         assert r["a"] == b"1"
@@ -351,6 +369,22 @@ class TestPipeline:
 
         assert r[key] == b"1"
 
+    def test_exec_error_in_pipeline_truncated(self, r):
+        key = "a" * 50
+        a_value = "a" * 20
+        b_value = "b" * 20
+
+        r[key] = 1
+        with r.pipeline(transaction=False) as pipe:
+            pipe.hset(key, mapping={"field_a": a_value, "field_b": b_value})
+            pipe.expire(key, 100)
+
+            with pytest.raises(redis.ResponseError) as ex:
+                pipe.execute()
+
+            expected = f"Command # 1 (HSET {key} field_a {a_value} field_b...) of pipeline caused error: "
+            assert str(ex.value).startswith(expected)
+
     def test_pipeline_with_bitfield(self, r):
         with r.pipeline() as pipe:
             pipe.set("a", "1")
@@ -372,7 +406,6 @@ class TestPipeline:
     @pytest.mark.onlynoncluster
     @skip_if_server_version_lt("2.0.0")
     def test_pipeline_discard(self, r):
-
         # empty pipeline should raise an error
         with r.pipeline() as pipe:
             pipe.set("key", "someval")
@@ -395,3 +428,13 @@ class TestPipeline:
             response = pipe.execute()
         assert response[0]
         assert r.get("foo") == b"bar"
+
+    @pytest.mark.onlynoncluster
+    def test_send_set_commands_over_pipeline(self, r: redis.Redis):
+        pipe = r.pipeline()
+        pipe.hset("hash:1", "foo", "bar")
+        pipe.hset("hash:1", "bar", "foo")
+        pipe.hset("hash:1", "baz", "bar")
+        pipe.hgetall("hash:1")
+        resp = pipe.execute()
+        assert resp == [1, 1, 1, {b"bar": b"foo", b"baz": b"bar", b"foo": b"bar"}]
